@@ -3,14 +3,13 @@ import numpy as np
 from math import *
 from scipy.stats import norm
 from scipy.special import erfinv
-#import QuantLib as ql
 import matplotlib.pyplot as plt
 import statsmodels.api as sm
 import scipy
 import scipy.stats as stats
 import os
 import pymysql
-import cx_Oracle
+#import cx_Oracle
 import pickle
 import sys
 import seaborn
@@ -31,10 +30,20 @@ import matplotlib as mpl
 # mpl.rcParams['axes.unicode_minus'] = False #解决保存图像是负号显示为方块问题
 #读入pickle文件
 def read_pickle(filename):
+    '''
+    功能：读入pickle文件
+    输入：pickle文件路径
+    输出：sql数据
+    '''
     with open(filename, 'rb') as f:
         sql_data = pickle.load(f)
     return sql_data
 def save_pickle(filename,data):
+    '''
+    功能：保存数据至pickle文件
+    输入：保存文件的路径，待保存的数据
+    输出：无
+    '''
     with open(filename,'wb') as f:
         '''
             保存pickle文件到指定目录，这里每次保存前先将pickle清空
@@ -44,10 +53,18 @@ def save_pickle(filename,data):
         pick.clear_memo()
         pick.dump(data)
 def read_feather(filename):
+    '''
+    功能：读取feather文件（后缀为.txt)
+    输入：feather文件路径
+    输出：feather文件中储存的object
+    '''
     # sql_data = feather.read_feather(filename)
     return pd.read_feather(filename)
 def save_feather(filename,data):
     """
+    功能：将数据保存在feather文件中
+    输入：保存文件的路径和数据
+    输出：无
     注意：要求存储内容保持一致
     """
     # feather.write_feather(data,filename)
@@ -85,6 +102,11 @@ def save_hdf5(df_or_dict,file_full_root,subgroup=None):
     print(file_full_root+' save finished!')
 # 读取hdf5文件，h5文件可以是里面['_data']是df，也可以是一整个dict{df},或者选定要读取哪个subgroup
 def read_hdf5(file_full_root,subgroup=None):
+    '''
+    功能：读取hdf5文件，h5文件可以是里面['_data']是df，也可以是一整个dict{df},或者选定要读取哪个subgroup
+    输入：文件路径，选定读取的subgroup(默认读整个）
+    输出：hdf5文件内的df或df字典信息
+    '''
     if os.path.exists(file_full_root):
         file_h = pd.HDFStore(file_full_root,'r')
         file_keys_raw = file_h.keys()
@@ -120,9 +142,11 @@ def read_hdf5(file_full_root,subgroup=None):
 # 从东吴在线落地数据库提取数据
 def getSql(sql_req):
     """
-
+    功能：从东吴在线落地数据库提取数据
+    输入：
     @param sql_req:sql代码
     @return:DF格式
+    输出：读取的sql信息
     """
     os.environ['NLS_LANG'] = 'SIMPLIFIED CHINESE_CHINA.UTF8'
     Oracle_userName = 'sjcj'
@@ -148,28 +172,41 @@ def get_value(df,n):
 #计算最大回撤
 def maxDrawDown(Rev_seq):
     """
-    求最大回撤率
-    #param return_list:Series格式月度收益率
-    #return：0~1
-
+    功能:求最大回撤率
+    输入:return_list:Series格式月度收益率
+    输出:最大回撤（值在0-1之间）
     """
     Rev_list=(Rev_seq+1).cumprod()
     return (Rev_list/Rev_list.cummax()-1).min()
 #修改——自动多样式
 def toTime(x):
+    '''
+    功能:将字符串格式转化为datetime格式
+    输入:包含日期字符串list或其他iterable格式
+    输出:包含日期datetime的序列
+    '''
     if(len(str(pd.Series(x).iloc[0]))==6):
         return pd.Series(x).apply(lambda x:pd.to_datetime(str(int(x))+'01'))
     return pd.Series(x).apply(lambda x:pd.to_datetime(str(int(x))))
+
 def fromTime(x,strtype='%Y%m%d'):
+    '''
+    功能：将日期格式转化为int格式
+    输入：包含日期格式的iterable格式,str格式
+    输出：包含int格式日期的序列
+    '''
     return pd.Series(x).apply(lambda x:int(x.strftime(strtype)))
+
 #回归：x可以选择是否加截距项
 def regress(y,x,con=True,weight=1):
     """
-    回归——注意不能出现空值
+    功能：回归——注意不能出现空值
+    输入：
     @param y:   应变量
     @param x:   自变量
     @param con: 是否加入截距项（涉及到行业信息的不加）
-    @return: 返回res  残差：res.resid
+    @param weight: 加权参数，默认为1
+    输出: 返回res  残差：res.resid
     """
     import statsmodels.api as sm # 最小二乘
     from statsmodels.stats.outliers_influence import summary_table # 获得汇总信息
@@ -180,6 +217,13 @@ def regress(y,x,con=True,weight=1):
     return res
 #更新数据 获得最新交易日历、计算收益序列数据  输出到一张表中：各指标最新日期
 def updateStockDailyRet():
+    '''
+    功能：更新数据，获得最新交易日历、计算收益序列数据
+    输入：无
+    输出：
+    @stockret.csv 更新后的数据表
+    @stockret.txt 更新后的数据feather文件
+    '''
     close=read_feather(Datapath+'BasicFactor_Close.txt').set_index('time')
     adjfactor=read_feather(Datapath+'BasicFactor_AdjFactor.txt').set_index('time')
     adjclose=close*adjfactor
@@ -207,12 +251,22 @@ def updateStockDailyRet():
     save_feather(Datapath+'stockret.txt',adjclose.reset_index(drop=True))
 #获得交易日历
 def getTradeDateList(period='date'):
+    '''
+    功能:获得交易日历
+    输入:交易周期，默认为日度，可修改为月度
+    输出:pandas series包含交易日期
+    '''
     data=read_feather(Datapath+'BasicFactor_close.txt')
     if(period=='date'):
         return data['time']
     elif(period=='month'):
         return pd.Series(data['time'].apply(lambda x:int(str(x)[:6])).unique(),name='time')
 def getFisicalList(period='Season'):
+    '''
+    功能：获得财务公告日历
+    输入：数据周期，默认为季度，可改为年度
+    输出：pandas series包含公告日期
+    '''
     data=read_feather(Datapath+'BasicFactor_AShareFinancialIndicator_ANN_DT.txt')
     year_list=pd.Series(data['time'].apply(lambda x:int(str(x)[:4])).unique(),name='time')
     if(period=='year'):
@@ -226,17 +280,30 @@ def getFisicalList(period='Season'):
         return monthlist[monthlist>=19891231]
 #获取股票序列
 def getStockList():
+    '''
+    功能：获取股票序列数据
+    输入：无
+    输出：pandas series包含数据库中所有股票
+    '''
     data=read_feather(Datapath+'BasicFactor_close.txt').set_index('time')
     return pd.Series(data.columns,name='code')
 #获取收益序列数据
 def getRetData():
+    '''
+    功能：获取收益学咧数据
+    输入：无
+    输出：pandas dataframe包含数据库中的股票收益
+    '''
     retData=read_feather(Datapath+'stockret.txt').dropna()
     return retData
 
 #读取指数行情数据 改 缺指数收益数据
 def getIndexData(code):
     """
-    code=["000300.SH",'000905.SH','000985.SH']
+    功能：获取指数行情数据
+    输入：股票代码
+    示例：code=["000300.SH",'000905.SH','000985.SH']
+    输出：指数行情数据
     """
     indexData=pd.read_csv(filepathtestdata+'index_data.csv',index_col=0)
     indexData.columns=['code','time','ret']
@@ -246,7 +313,10 @@ def getIndexData(code):
 #读取指数成分股列表 改  缺 500、1000成分股数据
 def getIndexComponent(indexname='wind'):
     """
-     indexname={300,500,800,1000}
+    功能：获取指数成分股列表
+    输入：指数名称，默认万得，可改为恒生300，中证500，恒生300+中证500，中证1000
+    indexname={300,500,800,1000}
+    输出：dataframe包含成份股列表信息
     """
     if(indexname=='wind'):
         IndexComponent=readLocalData('WINDAComponent.txt').dropna()
@@ -274,8 +344,9 @@ def getIndexComponent(indexname='wind'):
 #筛选股票 
 def setStockPool(DF,DFfilter):
     """
+    功能：将DFfilter中的股票在DF中筛选出来
     stockpool = ["000300.SH",'000905.SH','000985.SH']
-    参数：DF  columns=[code,date,xx]
+    返回：筛选后的DF  columns=[code,date,xx]
     """
     DF=DF.merge(DFfilter[['time','code']],on=['code','time'])
     return DF
@@ -283,9 +354,13 @@ def setStockPool(DF,DFfilter):
 #得到申万行业分类数据 
 def getSWIndustryData(level=1,freq='month',fill=False):
     """
-        startdate 格式例20200731  缺失则代表不加限制
+    功能：获取申万行业分类数据
+    输入：行业分类级数，默认为一级，可以改为二级
+        频率，默认为月
+        fill是否填充空值，默认为否
+    输出：dataframe包含时间、股票和对应行业分类信息
     """
-    ind_data=pd.read_feather(Datapath+r'\BasicFactor_Swind_Component.txt').set_index('time')
+    ind_data=pd.read_feather(Datapath+'BasicFactor_Swind_Component.txt').set_index('time')
     if(freq=='month'):
         ind_data=ind_data.pipe(applyindex,toTime).resample('m').last().pipe(applyindex,fromTime,'%Y%m')
         
@@ -304,7 +379,11 @@ def getSWIndustryData(level=1,freq='month',fill=False):
 #加上行业数据列 
 def addSWIndustry(DF,level=1,freq='month'):
     """
-        level 1 申万一级行业
+    功能：给数据dataframe加上行业数据列
+    输入：已有数据dataframe
+        行业分类级数，默认为一级，可改为二级
+        频率，默认为月
+    输出：加上申万行业数据的dataframe
     """
     SWData=getSWIndustryData(level,freq=freq)
     DF=DF.merge(SWData,on=['time','code'],how='left')
@@ -313,6 +392,11 @@ def addSWIndustry(DF,level=1,freq='month'):
 #加上板块信息
 
 def getSWSector(freq='month'):
+    '''
+    功能：获取申万板块信息
+    输入：数据频率
+    输出：申万板块数据dataframe
+    '''
     sector = pd.read_csv(filepathtestdata+'sw1.csv')[['申万代码','板块']]
     swind=getSWIndustryData(freq=freq).merge(sector, left_on='SWind', right_on='申万代码')
     swind=swind.drop(['SWind','申万代码'], axis=1).rename(columns={'板块': 'sector'})
@@ -324,8 +408,10 @@ def getSWSector(freq='month'):
 
 def addSWSector(DF, freq='month'):
     '''
-        DF: 三列标准型
+    功能：给现有数据dataframe加上申万板块数据
+    输入：DF: 三列标准型
         freq: 'day', 'month' 可选日频月频
+    输出：加上申万板块数据后的dataframe
     '''
     sector=getSWSector(freq=freq)
     DF = DF.merge(sector, on=['time', 'code'], how='left')
@@ -333,6 +419,11 @@ def addSWSector(DF, freq='month'):
 
 
 def getIndRetData():
+    '''
+    功能：获取行业收益信息
+    输入：无
+    输出：行业收益dataframe三列式
+    '''
     ret=read_feather(Datapath+'BasicFactorSW_S_dq_Close.txt').set_index('time')
     ret=ret.stack().reset_index()
     ret.columns=['time','code','ret']
@@ -369,8 +460,10 @@ def industryAggregate(IndInfo):
 #读取barra因子
 def getBarraData(startdate='',enddate='',freq='month'):
     """
-        startdate 格式例20200731 int
-        返回DataFrame  time code barra factor (注:日频)
+    功能：读取Barra因子
+    输入：开始日期和截止日期，startdate 格式例20200731 int
+         freq频率，默认为月频
+    输出：Barra因子DataFrame，列为 time code barra_factor (注:日频)
     """
     BarraData = read_hdf5(filepathtestdata + 'FactorLoading_Style.h5')
     BarraDataDF = pd.DataFrame([], columns=['time', 'code'])
@@ -395,8 +488,9 @@ def getBarraData(startdate='',enddate='',freq='month'):
 #模块转换——因子
 def factorStack(factor,factorname):
     '''
-        factor因子矩阵
-        factorname:因子名,string
+    功能：将因子数据dataframe堆叠转化
+    输入：factor因子矩阵，factorname:因子名,string
+    输出：转化后的因子数据
     '''
     factor=factor.stack().reset_index()
     factor.columns=['time','code',factorname]
@@ -405,7 +499,11 @@ def factorStack(factor,factorname):
 
 #剔除ST、上市60天以内、停牌股
 def kickout(mer):
-    # ST状态
+    '''
+    功能：剔除ST、上市60天以内、停牌股
+    输入：待剔除的数据dataframe
+    输出：剔除后的dataframe
+    '''
     ValidDF=read_feather(Datapath + 'BasicFactor_ValidDF.txt')
     ValidDF['time']=ValidDF['time'].apply(lambda x:int(str(x)[:6]))
     ValidDF=ValidDF.drop_duplicates(subset=['time'],keep='first').set_index('time').stack().reset_index()
@@ -416,49 +514,103 @@ def kickout(mer):
     return mer
 
 def zscore(x):
+    '''
+    功能：求series的z-score值
+    输入：pandas series
+    输出：z-score值
+    '''
     return (x-x.dropna().mean())/x.dropna().std()
 
 def zscorefac(x,fac):
-     x[fac]=(x[fac]-x[fac].dropna().mean())/x[fac].dropna().std()
-     return x
+    '''
+    功能：求dataframe数据中某因子的z-score值
+    输入：待测数据dataframe
+    输出：z-score改造后的数据dataframe
+    '''
+    x[fac]=(x[fac]-x[fac].dropna().mean())/x[fac].dropna().std()
+    return x
+
 def dePCT(x,fac,k1=0.01):
+    '''
+    功能：将数据dataframe中因子值上、下k分位数以外的值修改为上、下k分位数值
+    输入：数据dataframe，待修改的因子值列名称，分位数（默认为1%）
+    输出：修改极端值后的数据dataframe
+    '''
     x.loc[x[fac]>x[fac].quantile(1-k1),fac]=x[fac].quantile(1-k1)
     x.loc[x[fac]<x[fac].quantile(k1),fac]=x[fac].quantile(k1)
     return x
+
 def deSTD(x,fac,k1=2):
+    '''
+    功能：将数据dataframe中偏离平均值k倍标准差以上的值修改为偏离平均值k倍的值
+    输入：数据dataframe，待修改的因子值列名称，k倍数（默认为2）
+    输出：修改极端值后的数据dataframe
+    '''
     x.loc[x[fac]>x[fac].mean()+x[fac].std()*k1,fac]=x[fac].mean()+x[fac].std()*k1
     x.loc[x[fac]<x[fac].mean()-x[fac].std()*k1,fac]=x[fac].mean()-x[fac].std()*k1
     return x
 def deMAD(x,fac,k1=5):
+    '''
+    功能：将数据dataframe中偏离中位数值k倍MAD以上的值修改为偏离中位数值k倍MAD的值
+    输入：数据dataframe，待修改的因子值列名称，k倍数（默认为5）
+    输出：修改极端值后的数据dataframe
+    '''
     xmedian=x[fac].median()
     newmad=(x[fac]-xmedian).abs().median()
     x[fac]=np.clip(x[fac],xmedian-k1*newmad,xmedian+k1*newmad)
     return x
 
 def normBoxCox(x,fac):
+    '''
+    功能：将数据正态分布化
+    输入：数据dataframe和待修改的因子列名称
+    输出：修改后的数据dataframe
+    '''
     if(x[fac].dropna().count()>1):
         x[fac]=scipy.stats.boxcox(x[fac]-x[fac].min()+1)[0]
     return x
     
     
+
 def fillmean(x,fac):
+    '''
+    功能：以平均值填补数据dataframe中因子值列的空缺
+    输入：数据dataframe，待填补的因子值列名称
+    输出：修改后的dataframe
+    '''
     x[fac]=x[fac].fillna(x.mean())
     return x
 
 def fillmedian(x,fac):
+    '''
+    功能：以中位数值填补数据dataframe中因子值列的空缺
+    输入：数据dataframe，待填补的因子值列名称
+    输出：修改后的dataframe
+    '''
     x[fac]=x[fac].fillna(x.median())
     return x
 #因子初始化处理
+
 def factorInit(factor):
+    '''
+    功能：因子初始化处理，去除极端值
+    输入：待处理的因子数据
+    输出：处理后的因子数据
+    '''
     factor_list=getfactorname(factor,['time','code'])
     for facname in factor_list:
         factor=factor.groupby('time').apply(deMAD)
-        factor=factor.groupby('time').apply(zscore)
+        factor=factor.groupby('time').apply(zscore,facname)
     return factor
 
 
 #获得因子列表
 def getfactorname(x,L=['code','time']):
+    '''
+	功能：获取因子名称
+	输入：因子矩阵
+	返回：因子名称的list  ['Ret20',...]
+	'''
     factor_list=pd.Series(x.columns)
     for l in L:
         try:
@@ -470,8 +622,11 @@ def getfactorname(x,L=['code','time']):
 #日频转换为月频率
 def dayToMonth(DF,factor_list='',method='last'):
     """
-    @param x:
-    @param method:{'first','last','mean','sum'}
+    功能：将日频数据dataframe转化为月频
+    输入：
+        @param DF:待转化的数据dataframe
+        @param method: 保留数据（月末值，平均值，当月总和）{'last','mean','sum'}
+    输出：转化为月频后的数据dataframe
     """
     x=DF.copy()
     if(factor_list==''):
@@ -502,6 +657,11 @@ def dayToMonth(DF,factor_list='',method='last'):
 
 #多空t分组,并判断各股票的指标在哪一组
 def isinGroupT(x,factor_name,asc=True,t=5):
+    '''
+    功能：判断各股票的指标在多空分中的哪一组
+    输入：数据dataframe，因子名称，升/降序，分组组数
+    输出：因子指标转化为分组组数后的数据dataframe
+    '''
     x[factor_name]=pd.qcut(x[factor_name], t,labels=False,duplicates='drop')+1
     if(asc==False):
         x[factor_name]=t-x[factor_name]+1
@@ -509,45 +669,65 @@ def isinGroupT(x,factor_name,asc=True,t=5):
 
 #判断各股票的指标是否在前k（k%）
 def isinTopK(x,factor_name,asc=True,k=30):
+    '''
+    功能：判断各股票的指标知否在前k
+    输入：数据dataframe，因子名称，升/降序，前k数量（默认为30）
+    输出：因子指标转化为判断是否在前k（1表示在，0表示不在）后的数据dataframe
+    '''
     num = int(x.shape[0] * k) if k < 1 else k
-    if(x.shape[0]<k):
-        x[factor_name]=0
+    if(x.shape[0] < k):
+        x[factor_name] = 0
         return x
-    x[factor_name] = x[factor_name].rank(ascending=asc)
+    x[factor_name] = x[factor_name].rank(ascending = asc)
     x[factor_name] = x[factor_name].apply(lambda y: 1 if y <= num else 0)
     return x
 
 #组合业绩评估
 def calcGroupRet(x,factor_name,RetData=getRetData(),MVweight=False):
+    '''
+    功能：计算组合收益
+    输入：数据dataframe，因子名称，收益数据，是否按市值加权
+    输出：组合收益dataframe，索引为时间，一列为因子名
+    '''
     if(not('ret' in x.columns)):
-        x=x.merge(RetData,on=['time','code'])
+        x = x.merge(RetData,on = ['time','code'])
     if(MVweight==False):
         y = x.groupby(['time', factor_name])['ret'].mean().reset_index()
     else:
-        x=addXSize(x,norm='dont')
+        x = addXSize(x,norm = 'dont')
         y = x.groupby(['time', factor_name]).apply(lambda x:calcWeightedMean(x['ret'],x['CMV'])).reset_index()
         
-    y = y.pivot(index='time', columns=factor_name)
+    y = y.pivot(index = 'time', columns = factor_name)
     y.columns = y.columns.droplevel()
     return y
 
 
 #计算胜率、赔率
 def calcWRPL(x):
+    '''
+    功能：计算胜率、赔率
+    输入：包含收益数据ret列的数据dataframe
+    输出：包含胜率赔率两个数据的序列
+    '''
     return pd.Series([x[x.ret>0].count().ret*1.0/x.count().ret,x[x.ret>0].mean().ret/x[x.ret<0].mean().ret*-0.1],index=['WR','PL'])
-def calcGroupWR(x,by='group',Retdata=getRetData()):
-    xname=x.name
+
+def calcGroupWR(x,by='group',RetData=getRetData()):
+    '''
+    功能：计算组合胜率和赔率
+    输入：数据dataframe,
+    '''
+    xname = x.name
     if(not('ret' in x.columns)):
-        x=x.merge(RetData,on=['time','code'],how='outer')
+        x = x.merge(RetData, on=['time', 'code'], how='outer')
     x['ret']=x['ret']-x['ret'].median()
-    x=x[x[by]!=0]
-    t=len(x[by].unique())
-    if(t==0):
+    x = x[x[by] != 0]
+    t = len(x[by].unique())
+    if (t == 0):
         return 0
-    if(t==1):
-        groupWeight=[1]
+    if (t == 1):
+        groupWeight = [1]
     else:
-        groupWeight=[-1+i*(2.0/(t-1)) for i in range(t)]
+        groupWeight = [-1+i*(2.0/(t-1)) for i in range(t)]
     Ans=x.groupby(by).apply(calcWRPL).sort_index(ascending=False)
     Ans['w']=groupWeight
     Ans.loc[Ans.w<0,'WR']=1-Ans.loc[Ans.w<0,'WR']
@@ -560,9 +740,12 @@ def calcGroupWR(x,by='group',Retdata=getRetData()):
 #取残差
 def calcResid(y,x,intercept=True,retBeta=False):
     '''
+    功能：求回归残差
+    输入：
        x、y必须是DataFrame 格式，不能是Series
        用公式快速求解，需要更多信息用FB.regress
        retBeta='True'
+    输出：储存回归残差信息的序列
     '''    
     x=pd.DataFrame(x).reset_index(drop=True)
     y=pd.DataFrame(y).reset_index(drop=True)
@@ -581,16 +764,52 @@ def calcResid(y,x,intercept=True,retBeta=False):
 
 #加入sw行业
 def addXSWindDum(DF,freq='month'):
+    '''
+    功能：加入月度申万行业信息
+    输入：待修改的数据dataframe（月频）
+    输出：加入后的数据dataframe
+    '''
     ind_tmp=getSWIndustryData(freq=freq)
     if('SWind' in DF):
         del DF['SWind']
     DF=DF.merge(ind_tmp,on=['time','code'],how='left')
     return DF
 
+#获取中信行业数据,待完善
+def getZXIndustryData(level = 1, freq = "month",fill = False):
+    """
+        功能：获取中信行业分类数据
+        输入：行业分类级数，默认为一级，可以改为二级和三级
+            频率，默认为月
+            fill是否填充空值，默认为否
+        输出：dataframe包含时间、股票和对应行业分类信息
+    """
+    ind_data = pd.read_feather(Datapath + '').set_index('time')
+    if (freq == 'month'):
+        ind_data = ind_data.pipe(applyindex, toTime).resample('m').last().pipe(applyindex, fromTime, '%Y%m')
+    ind_data = ind_data.stack().reset_index()
+    ind_data.columns = ['time', 'code', 'ZX']
+    if (level == 1):
+        ind_data['ZX'] = ind_data['ZX'].apply(lambda x: str(x)[:2])
+    if (level == 2):
+        ind_data['ZX'] = ind_data['ZX'].apply(lambda x: str(x)[:4])
+    if (level == 3):
+        ind_data['ZX'] = ind_data['ZX'].apply(lambda x: str(x)[:6])
+    if (fill == False):
+        return ind_data
+    ind_data = ind_data.sort_values(by='time')
+    ind_data = ind_data.pivot(index='time', columns='code', values=['ZX'])
+    ind_data = ind_data.reindex(getTradeDateList()).fillna(method='ffill').stack().reset_index()
+    return ind_data
 
 #加入中信行业
 def addXZXind(DF,freq='month'):
-    ind_tmp=getZXIndustryData(freq=freq)
+    '''
+    功能：将中信行业数据加入现有dataframe
+    输入：现有标准三列dataframe
+    输出：加入中信行业数据后的四列dataframe
+    '''
+    ind_tmp=getZXIndustryData(level=1,freq=freq,fill = False)
     if('ZXind' in DF):
         del DF['ZXind']
     DF=DF.merge(ind_tmp,on=['time','code'],how='left')
@@ -598,7 +817,12 @@ def addXZXind(DF,freq='month'):
 
 
 def getCMV(freq='month'):
-    Size_data=pd.read_feather(Datapath+r'\BasicFactor_DqMV.txt')
+    '''
+    功能：获取市值数据
+    输入：数据频率（默认为月）
+    输出：标准三列dataframe包含市值数据
+    '''
+    Size_data=pd.read_feather(Datapath+'BasicFactor_DqMV.txt')
     if(freq=='month'):
         Size_data['time']=Size_data['time'].apply(lambda x:int(str(x)[:6]))
         Size_data=Size_data.fillna(method='ffill').drop_duplicates(['time'],keep='last')
@@ -609,24 +833,9 @@ def getCMV(freq='month'):
 
 def addXSize(DF,freq='month',norm='boxcox'):
     '''
-    
-
-    Parameters
-    ----------
-    DF : TYPE
-        DESCRIPTION.
-    freq : TYPE, optional
-        DESCRIPTION. The default is 'month'.
-    norm : TYPE, optional
-        DESCRIPTION. The default is 'boxcox'. 
-        boxcox 标准化
-        dont 不用做标准化（流通市值加权用）
-
-    Returns
-    -------
-    TYPE
-        DESCRIPTION.
-
+    功能：加入月度规模信息
+    输入：待修改的标准三列数据dataframe,freq默认月频，norm默认为boxcox标准化，可选dont不做标准化
+    输出：加入后的数据四列dataframe
     '''
     Size_data=getCMV(freq=freq)
     def boxcoxcmv(x):
@@ -641,8 +850,14 @@ def addXSize(DF,freq='month',norm='boxcox'):
         del DF['CMV']
     DF=DF.merge(Size_data,on=['time','code'],how='left')
     return DF
+
+
 def RegbyindSize(x,name):
-    #代码补齐
+    '''
+    功能：计算现有因子和行业市值因子的回归残差，用于中性化
+    输入：因子信息，因子名称，因子列表
+    输出：回归残差
+    '''
     def zscore(x):
         return (x-x.dropna().mean())/x.dropna().std()
     x_tmp=x[['time','code',name,'SWind','CMV']]
@@ -663,6 +878,11 @@ def RegbyindSize(x,name):
 
 #计算行业市值中性
 def calcNeuIndSize(factor,factor_name,freq='month'):
+    '''
+    功能：计算行业市值中性，排除行业市值对因子的影响
+    输入：因子信息和因子名称，评率默认为月，可选日
+    输出：中性化后的因子信息
+    '''
     factor_tmp=factor.copy()
     if('CMV' not in factor):
         factor_tmp=addXSize(factor_tmp,freq=freq)
@@ -679,7 +899,11 @@ def calcNeuIndSize(factor,factor_name,freq='month'):
 
 
 def RegbySize(x,name):
-    #代码补齐
+    '''
+    功能：计算现有因子和行业市值因子的回归残差，用于中性化
+    输入：因子信息，因子名称
+    输出：回归残差
+    '''
     def zscore(x):
         return (x-x.dropna().mean())/x.dropna().std()
     x_tmp=x[['time','code',name,'CMV']]
@@ -697,6 +921,11 @@ def RegbySize(x,name):
 
 #计算市值中性
 def calcNeuSize(factor,factor_name,freq='month'):
+    '''
+    功能：计算市值中性，排除市值对因子的影响
+    输入：因子信息和因子名称，数据频率（默认为月）
+    输出：中性化后的因子信息
+    '''
     factor_tmp=factor.copy()
     if('CMV' not in factor):
         factor_tmp=addXSize(factor_tmp,freq=freq)
@@ -710,8 +939,12 @@ def calcNeuSize(factor,factor_name,freq='month'):
 
 
 
-
 def addXBarra(DF,Barra_list='',freq='month'):
+    '''
+    功能：加入Barra因子数据
+    输入：待修改的数据dataframe，barra因子列表，数据频率（默认为月度）
+    输出：加入后的数据dataframe
+    '''
     barra_tmp=getBarraData(freq=freq)
     if(Barra_list==''):
         Barra_list=getfactorname(barra_tmp)        
@@ -719,7 +952,13 @@ def addXBarra(DF,Barra_list='',freq='month'):
         Barra_list=[Barra_list]
     DF=DF.merge(barra_tmp[['time','code']+Barra_list],on=['time','code'],how='left')
     return DF,Barra_list
+
 def RegbyBarra(x,name,factor_list):
+    '''
+    功能：计算现有因子和barra因子的回归残差，用于中性化
+    输入：因子信息，barra因子名称，因子列表
+    输出：回归残差
+    '''
     #代码补齐
     def zscore(x):
         return (x-x.dropna().mean())/x.dropna().std()
@@ -742,6 +981,11 @@ def RegbyBarra(x,name,factor_list):
 
 #纯因子
 def calcNeuBarra(factor,factor_name,factor_list='',freq='month'):
+    '''
+    功能：计算Barra因子中性，排除barra因子对待处理因子的影响
+    输入：因子信息，因子名称，barra因子列表，数据频率（默认为月）
+    输出：中性化后的因子信息
+    '''
     factor_tmp,factor_list=addXBarra(factor,Barra_list=factor_list,freq=freq)
     if('SWind' not in factor):
         factor_tmp=addXSWindDum(factor_tmp,freq=freq)
@@ -753,8 +997,12 @@ def calcNeuBarra(factor,factor_name,factor_list='',freq='month'):
     factor=factor.merge(factor_tmp[['time','code',factor_name+'_pure']],on=['time','code'])
     return factor
 
-#计算IC ——spearmanr 方法
 def calcIC(mer, factor_name):
+    '''
+    功能：用pearsonr和spearmanr方法计算相关性（IC、rankIC）
+    输入：包含因子值和回报的数据dataframe，因子名称
+    输出：包含IC,ICIR,rankIC,rankICIR,t_value的序列
+    '''
     sp1 = mer.groupby('time').apply(lambda x: stats.pearsonr(x['ret'], x[factor_name])[0])
     IC = sp1.mean()
     ICIR = (sp1.mean()) / sp1.std() * 12 ** 0.5
@@ -771,7 +1019,11 @@ def calcIClist(DF):
     IC = sp.mean()
     ICIR =x
 
-def ZipLocalFiles(file_path,save_path,zipname='',t=5):#t是分成的压缩包数量默认为5,file_path是需要压缩的文件夹路径,save_path是储存路径
+def ZipLocalFiles(file_path,save_path,zipname='',t=5):
+    '''
+    功能：将本地文件打包
+    输入：t是分成的压缩包数量默认为5,file_path是需要压缩的文件夹路径,save_path是储存路径
+    '''
     def zip_files(file_path,files, zip_name ):
         zip = zipfile.ZipFile( zip_name, 'w', zipfile.ZIP_DEFLATED )
         for file in files:
@@ -810,7 +1062,11 @@ def ZipLocalFiles(file_path,save_path,zipname='',t=5):#t是分成的压缩包数
         
 
 
-def copyFile(file_path,target_path):                
+def copyFile(file_path,target_path):
+    '''
+    功能：复制本地文件
+    输入：待复制文件路径，文件粘贴路径
+    '''
     if not os.path.isfile(file_path):
         print ("%s 不存在!"%(file_path))
     else:
@@ -821,20 +1077,22 @@ def copyFile(file_path,target_path):
 
 
 def _calcGrowthRate(x,t=3):
+    '''
+    功能：利用最小二乘法回归计算年成长率
+    输入：x为矩阵形式数据，t为窗口年数
+    输出：回归得到的成长率值
+    '''
     ols=po.PandasRollingOLS(y=pd.DataFrame(x),x=pd.DataFrame(np.arange(1, len(x)+1)),use_const=True,window=t)
     return pd.Series(ols.beta[0]/x.abs().rolling(window=t).mean(),name=x.name)
-#计算过去windows年成长率
+
+
+
 def calcGrowthRate(data,window=3):
     '''
-    计算年化复合变化率  (同比)
+    功能：计算过去windows年复合变化率（同比）
     计算公式 regress(data,(1,2,...),intercept=True).beta / y.abs().mean()
-    Parameters
-    ----------
-    data:矩阵形式
-    window ：窗口期（年）默认为3
-    -------
-    返回矩阵，每一个值为变化率
-
+    输入：data为矩阵形式，window为窗口期（年）（默认为3）
+    输出：数据矩阵，每一个值为变化率
     '''
     
     tmp=data.dropna(how='all').reset_index()
@@ -848,7 +1106,11 @@ def calcGrowthRate(data,window=3):
     tmp=pd.concat(ans).sort_index()
     return tmp
 
-def copyFiles(A,B,cover=True):#A为源文件路径B为新文件路径
+def copyFiles(A,B,cover=True):
+    '''
+    功能：复制文件夹
+    输入：A为源文件路径，B为新文件路径
+    '''
     source_path= os.path.abspath(A)
     target_path = os.path.abspath(B)
     if cover:       
@@ -872,6 +1134,9 @@ def copyFiles(A,B,cover=True):#A为源文件路径B为新文件路径
                     shutil.copytree(dir_file,target_path+'//'+dirss)
 
 def calcAllFactorAns():
+    '''
+    功能：计算所有储存在factorInfo文件中的因子的收益、IC等信息
+    '''
     import FactorTest.FactorTestMain as FM
     datainfo=pd.read_excel(FactorInfopath)
     for i in tqdm(datainfo.index):
@@ -892,9 +1157,11 @@ def calcAllFactorAns():
 
 def calcPortfolioRet(Rev_seq,t=12):
     """
-    @param Rev_seq:
-    @param t:天数 默认12(月) 252 日
-    @return:
+    功能：计算组合收益率
+    输入：
+        @param Rev_seq:组合收益序列
+        @param t:天数 默认12(月) 252 日
+    输出：组合收益序列数据，包含年化收益率，信息比率，胜率，最大回撤
     """
     ret_mean=e**(Rev_seq.apply(lambda x:np.log(x+1)).mean()*t)-1
     ret_sharpe=Rev_seq.mean()*t/Rev_seq.std()/t**0.5
@@ -905,6 +1172,11 @@ def calcPortfolioRet(Rev_seq,t=12):
 
 #计算换手率
 def calcAnnualTurnover(GroupData,facname,t=12):
+    '''
+    功能：获取年度换手率信息
+    输入：三列标准dataframe，因子名，计算周期
+    输出：各个因子下的年度换手率
+    '''
     groupdata=GroupData
     groupNum = groupdata[['time', 'code', facname]].groupby(['time', facname]).count().reset_index()
     groupNum['weight'] = 1 / groupNum['code']
@@ -923,6 +1195,9 @@ def calcAnnualTurnover(GroupData,facname,t=12):
 
 # 更新数据库信息
 def DataRenewTime():
+    '''
+    功能：更新dataInfo.csv中的信息（最新数据时间）
+    '''
     datainfo = pd.read_excel(DataInfopath)
     for i in tqdm(datainfo['存储地址'].unique()):
         if (os.path.exists(Datapath + i)):
@@ -930,7 +1205,11 @@ def DataRenewTime():
         else:
             datainfo.loc[datainfo['存储地址']==i, '最新时间'] = np.nan
     datainfo.to_excel(DataInfopath, index=False)
+
 def FactorRenewTime():
+    '''
+    功能：更新因子库（FactorInfo.xlsx)信息（最新因子数据时间）
+    '''
     datainfo = pd.read_excel(FactorInfopath)
     for i in datainfo.index:
         if (os.path.exists(Factorpath + datainfo.loc[i, '地址'])):
@@ -938,14 +1217,13 @@ def FactorRenewTime():
         else:
             datainfo.loc[i, '最新时间'] = np.nan
     datainfo.to_excel(FactorInfopath, index=False)
-  
+
 # 更新factor最新信息
-# 获得开始更新时间
 def getUpdateStartTime(x, backdays=0):
     '''
-      backdays是回退天数
-      日频使用1日
-      更低频率anndt等使用5日
+    功能：获得开始更新时间
+    输入：x为infoDF中的最新时间，backdays是回退天数，日频使用1日，更低频率anndt等使用5日
+    输出：开始更新时间
     '''
     if(type(x)!=pd.core.series.Series):
         x=pd.Series(x)
@@ -958,7 +1236,9 @@ def getUpdateStartTime(x, backdays=0):
 
 def readLocalData(rawpath,key=''):
     """
-       param:path 路径  key 键名
+    功能：读取本地数据，不需要准备Datapath， 读入自动转化为['time','code',key]
+    输入：数据路径，键名
+    输出：数据dataframe
     """
     path=Datapath+rawpath
     if(os.path.exists(path)):
@@ -976,6 +1256,11 @@ def readLocalData(rawpath,key=''):
 
 #批量读入
 def readLocalDataSet(path_list):
+    '''
+    功能：批量读入信息
+    输入：待读入的文件路径列表['A','B','C']
+    输出：读入的信息dataframe
+    '''
     data_tot=pd.DataFrame(index=['time','code']).T
     for path in path_list:
        data_tot=data_tot.merge(readLocalData(path),on=['time','code'],how='outer') 
@@ -983,12 +1268,12 @@ def readLocalDataSet(path_list):
 
 #合并新旧信息（废弃）
 def mergeAB(rawData,newData):
-    """
     """
     #检查新信息最新行的空值率
     #    if(newData[newData.time==newData.time.max()].groupby('time').count().iloc[-1,-1]==0):
     #        print( Exception('最后一行为空'))
     #目前采用后值覆盖方法，假定后值正确
+    """
     if(rawData.shape[0]>0):
         rawData0=rawData[rawData.time<newData['time'].min()]
         rawData=rawData[rawData.time>=newData['time'].min()]
@@ -999,6 +1284,11 @@ def mergeAB(rawData,newData):
     return rawData
 
 def readLocalFeather(path):
+    '''
+    功能：读取本地feather文件，无需加入Datapath
+    输入：feather文件路径
+    输出：包含feather文件信息的dataframe
+    '''
     if(os.path.exists(Datapath+path)):
         return read_feather(Datapath+path)
     else:
@@ -1007,6 +1297,11 @@ def readLocalFeather(path):
 
 #用于存储sql型数据  以object_ID为单位
 def saveSqlData(sqlData,infoDF):
+    '''
+    功能：储存sql型数据到infoDF中
+    输入：sql型数据和infoDF
+    输出：无
+    '''
     path=Datapath+infoDF.loc[infoDF.index[0],'存储地址']
     if(os.path.exists(path)):
         data=read_feather(path)
@@ -1019,9 +1314,15 @@ def saveSqlData(sqlData,infoDF):
     data=data.append(sqlData).drop_duplicates(subset=['id'],keep='last').sort_values(by='time')
     data0=data0.append(data)
     save_feather(Datapath+infoDF.loc[infoDF.index[0],'存储地址'],data0)
+
 #存储日频数据
 def saveDailyData(sqlData,infoDF):
-     for i in infoDF.index:
+    '''
+    功能：储存日频数据为feather文件
+    输入：sql数据和因子信息dataframe
+    输出：无
+    '''
+    for i in infoDF.index:
         sql_data1=sqlData[['time','code',infoDF.loc[i,'数据库键']]].pivot(index='time',columns='code',values=infoDF.loc[i,'数据库键']).reset_index()
         data0=readLocalFeather(infoDF.loc[i,'存储地址'])
         data0=data0.append(sql_data1)
@@ -1032,7 +1333,12 @@ def saveDailyData(sqlData,infoDF):
 
 #用于存储财务数据
 def saveFinData(sqlData,infoDF):
-     for i in tqdm(infoDF.index):
+    '''
+    功能：储存财务数据为feather文件
+    输入：sql数据和因子信息dataframe
+    输出：无
+    '''
+    for i in tqdm(infoDF.index):
         sql_data1=sqlData[['time','code',infoDF.loc[i,'数据库键']]].drop_duplicates(subset=['time','code'],keep='last').pivot(index='time',columns='code',values=infoDF.loc[i,'数据库键'])
         data0=readLocalFeather(infoDF.loc[i,'存储地址']).set_index('time',drop=True)
         data0=data0.append(sql_data1).reset_index()
@@ -1043,7 +1349,12 @@ def saveFinData(sqlData,infoDF):
         save_feather(Datapath+infoDF.loc[i,'存储地址'],data0.set_index('time').reset_index())
 #用于存储行业成分股数据
 def saveIndData(sqlData,infoDF):
-     for i in infoDF.index:
+    '''
+    功能：储存行业数据为feather文件
+    输入：sql数据和因子信息dataframe
+    输出：无
+    '''
+    for i in infoDF.index:
          sqlData1=sqlData[['time','code',infoDF.loc[i,'数据库键']]]
          sqlData2=sqlData[['eddate','code']].dropna()
          sqlData2.columns=['time','code']
@@ -1063,6 +1374,11 @@ def saveIndData(sqlData,infoDF):
          
 #存储指数成分股
 def saveIndexComponentData(sqlData,infoDF):
+    '''
+    功能：储存指数成分股为feather文件
+    输入：sql数据和因子信息dataframe
+    输出：无
+    '''
     i=infoDF.index[0]
     sqlData.columns=['time','code','signal']        
     sqlData['time']=sqlData['time'].apply(lambda x:x.strftime('%Y%m%d'))
@@ -1077,6 +1393,11 @@ def saveIndexComponentData(sqlData,infoDF):
 
 
 def toShortForm(DF,faclist=''):
+    '''
+    功能：将dataframe转化为time，code两列的类数据库形式
+    输入：数据dataframe
+    输出：转化后的数据dataframe
+    '''
     if(faclist==''):
         faclist=getfactorname(DF,['code','time'])
     if(type(faclist)=='str'):
@@ -1085,13 +1406,23 @@ def toShortForm(DF,faclist=''):
     return DF1
 
 def toLongForm(DF,facname=''):
+    '''
+    功能：将dataframe进行堆叠，列旋转到行
+    输入：数据dataframe
+    输出：转化后的数据dataframe
+    '''
     DF=DF.stack().reset_index()
     if(facname!=''):
-        DF.columns=['time','code',factorname]
+        DF.columns=['time','code',facname]
     return DF
 
 
 def fillFisicalMonth(DF,faclist):
+    '''
+    功能：用财务月对数据Dataframe进行索引，用法：DF.pipe(FB.fillFisicalMonth,faclist)
+    输入：数据dataframe,因子列表
+    输出：用财务月作索引的数据dataframe
+    '''
     if(type(faclist)=='str'):
         faclist=[faclist]
     DF1=DF.drop_duplicates(subset=['time','code'],keep='last').pivot(index='time',columns='code',values=faclist).reindex(index=getTradeDateList('month'))
@@ -1099,13 +1430,13 @@ def fillFisicalMonth(DF,faclist):
 
 
 def partition(ls, size):
-    """
-    Returns a new list with elements
-    of which is a list of certain size.
-
+    '''
+    功能：将列表切片再合成为固定长度的嵌套列表
+    输入：待处理列表和固定长度
+    输出：处理后的嵌套列表
         >>> partition([1, 2, 3, 4], 3)
         [[1, 2, 3], [4]]
-    """
+    '''
     return [ls[i:i+size] for i in range(0, len(ls), size)]
          
          
@@ -1127,10 +1458,13 @@ def testTime(func):
 
 def transData(data,key='factor',ANN_DT='BasicFactor_AShareFinancialIndicator_ANN_DT.txt', output='', startmonth=199912, endmonth=210012):
     '''
-    即将更名transFisicalData函数
-    data为因子矩阵,需预先getFisicalList()处理
-    key因子名,output为输出形式（三列或矩阵型）
-    ANN_DT为公布日期文件地址，默认'BasicFactor_AShareFinancialIndicator_ANN_DT.txt'；可选三大表ANN_DT
+    功能：将财务数据转化为时间、股票代码、因子名称三列dataframe或矩阵
+        即将更名transFisicalData函数
+    输入：
+        data为因子矩阵,需预先getFisicalList()处理
+        key因子名,output为输出形式（三列或矩阵型）
+        ANN_DT为公布日期文件地址，默认'BasicFactor_AShareFinancialIndicator_ANN_DT.txt'；可选三大表ANN_DT
+        startmonth和endmonth为int型，表示开始和结束月份
     '''
 
     ANN_DT=read_feather(Datapath + ANN_DT).set_index('time').reindex(getFisicalList())
@@ -1171,13 +1505,20 @@ def transFisicalData(data,key='factor',ANN_DT='BasicFactor_AShareFinancialIndica
 
 
 def calc_plot(DF):
+    '''
+    功能：对dataframe进行plot画图
+    输入：用于画图的dataframe
+    '''
     # DF=DF.reset_index()
     # DF['time']=DF['time'].apply(toTime)
     # DF.set_index('time').plot()
     applyindex(DF,lambda x:str(x)).plot()
+
 def calcFisicalLYR(DF):
     '''
-    矩阵型  转为上年末数据
+    功能：将3、6、9月末财务数据时间统一为上年末数据
+    输入：三列标准矩阵
+    输出：修改后的三列标准矩阵
     '''
     DF=DF.loc[getFisicalList('year')].fillna('empty').reindex(getFisicalList()).ffill()
     DF[DF=='empty']=np.nan
@@ -1185,7 +1526,9 @@ def calcFisicalLYR(DF):
 
 def calcFisicalq(DF):
     '''
-    矩阵型  转为单季度数据
+    功能：将6、9、12月末的当年累积时段财务数据转化为当季度内的数据
+    输入：三列标准矩阵
+    输出：修改后的三列标准矩阵
     '''
     DFlast=DF.copy()
     DFlast.loc[getFisicalList('year')[getFisicalList('year').isin(DFlast.index)]]=0
@@ -1194,14 +1537,18 @@ def calcFisicalq(DF):
 
 def calcFisicalttm(DF):    
     '''
-    矩阵型  转为ttm数据
+    功能：将季度时段财务数据转化为过去四个季度的和（ttm数据）
+    输入：三列标准矩阵
+    输出：修改后的三列标准矩阵
     '''
     DFq=calcFisicalq(DF)
     return DFq.rolling(window=4).sum()
 
 def applyindex(x,func,*args,**kwargs):
     '''
-       直接对DataFrame或Series 对index执行func
+    功能：直接对DataFrame或Series的index执行func
+    输入：x:dataframe或series， func，*args，**kwargs：要执行的function和辅助参数
+    输出：修改完成的dataframe或series
     '''
     if(x.index.name==None):
         x.index.name='index'
@@ -1209,20 +1556,34 @@ def applyindex(x,func,*args,**kwargs):
     x=x.reset_index()
     x[xname]=x[xname].apply(lambda x:func(x,*args,**kwargs))
     return x.set_index(xname)
+
 # 半衰期序列
 def calc_exp_list(window,half_life):
+    '''
+    功能：根据因子半衰期生成权重序列
+    输入：window为int表示窗口期，half_life为半衰期序列
+    输出：由半衰期计算的权重序列
+    '''
     exp_wt = np.asarray([0.5 ** (1 / half_life)] * window) ** np.arange(window)
     return exp_wt[::-1] / np.sum(exp_wt)
 
-
 #weighted_std
 def calcWeightedStd(series, weights):
     '''
-       加权平均
+    功能：计算加权标准差
+    输入：待计算序列和权重
+    输出：加权标准差值
     '''
     weights /= np.sum(weights)
     return np.sqrt(np.sum((series-np.mean(series)) ** 2 * weights))
+
+
 def calcWeightedMean(series,weight):
+    '''
+    功能：计算加权平均
+    输入：待计算序列和权重
+    输出：加权平均值
+    '''
     return np.sum(series*weight)/np.sum(weight)
 
 #滚动回归(待完善)
@@ -1235,18 +1596,9 @@ def rollingRegress(y,x,window,const=True):
         
 def monthToDay(DF,factor_list=''):
     '''
-
-    Parameters
-    ----------
-    DF : 
-        三列标准型.
-    factor_list : list, optional
-        需要转换的因子. The default is ''.
-
-    Returns
-    -------
-    三列标准型.
-
+·   功能：将月频数据dataframe转化为日频数据dataframe，空缺数据向下补齐
+    输入：DF为原月频数据dataframe（三列标准型），fator_list为需要转换的因子列表
+    输出：改造后的数据dataframe（三列标准型）
     '''
     x=DF.copy()
     if(factor_list==''):
@@ -1273,10 +1625,12 @@ def monthToDay(DF,factor_list=''):
 
 
 
-#画图模块
-
-
 def plotICList(IClist,RankICList,facname=''):
+    '''
+    功能：对多空组合的IC和rank IC进行绘图
+    输入：IClist月度IC列表
+        RankIClist月度IC列表
+    '''
     IC_df = pd.DataFrame(IClist,columns=['IC'])
     RankIC_df = pd.DataFrame(RankICList,columns=['RankIC'])
     plt.figure(figsize=(15,10))
@@ -1299,20 +1653,13 @@ def plotICList(IClist,RankICList,facname=''):
 
 def plotPortfolioList(portfolioList,facname='',ascloc=False,t='',is_sub_axis=1,lsname='多空组合'):
     '''
-    
-
-    Parameters
-    ----------
-    portfolioList : self.portfolioList[faclist]
-    facname : 表头名称对应因子
-    ascloc : TYPE, optional
-        DESCRIPTION. The default is False.
-    is_sub_axis : 是否需要右轴显示
-    lsname : 右轴显示的列  list表示多列，str表示单列
-
-    Returns
-    -------
-    None.
+    功能：对t分组的每组和多空组合的收益画图
+    输入：
+        portfolioList : 含有t分组和多空组合收益信息的dataframe
+        facname : 表头名称对应因子
+        ascloc ：升序或降序，默认为因子值小的为第一组
+        is_sub_axis : 是否需要右轴显示
+        lsname : 右轴显示的列  list表示多列，str表示单列
 
     '''  
     data = portfolioList.apply(lambda x:x+1).cumprod().pipe(applyindex,lambda x:str(x))
@@ -1361,13 +1708,17 @@ def plotPortfolioList(portfolioList,facname='',ascloc=False,t='',is_sub_axis=1,l
 
 
 class dataProcess():
+    '''
+      数据处理部分，嵌套在FM.FactorTest类中执行
+      只改变['v']取值——同步改变FactorTest类中的变量
+    '''
     def __init__(self,DataBase={'v':pd.DataFrame(columns=['time','code'])}):
         self.dataBase=DataBase
         self.datalist=[]
         self.latestname='all'
         pd.options.mode.use_inf_as_na = True  #剔除inf
 
-    #返回最新值self. latest
+    #返回最新值self.latest
     def __call__(self,name='last'):
         if(name=='last'):
             name=self.latestname
@@ -1375,6 +1726,7 @@ class dataProcess():
             return self.dataBase['v'][['time','code',name]]
         else:
             return self.dataBase['v']
+
     #因子处理_等待废弃
     def updateData(self,DataBase):
         self.dataBase=DataBase
@@ -1406,6 +1758,11 @@ class dataProcess():
             
     #横截面回归取残差，速度慢尽量月频
     def AregBResid(self,yname,xname,rname='resid'):
+        '''
+          yname 因变量 xname自变量  str /list均可
+          返回回归残差，列名为rname
+          自动放回dataBase['v']中
+        '''
         if(type(yname)==str):
             yname=[yname]
         if(type(xname)==str):
@@ -1421,13 +1778,17 @@ class dataProcess():
         self.tmp=Ans
         self.dataBase['v']=pd.concat(Ans)
         self.latestname=rname
-    def addFactors(self,addList='',factorname='factorSum'):
-        '''
 
+    #因子合成——待完善
+    def addFactors(self,addList='',weight_list='',factorname='factorSum'):
+        '''
+        因子中性化等权合成 
         Parameters
         ----------
         addList : list
-            需要相加的因子列表.
+            需要相加的因子列表
+        weight_list: Series   index addList  columns:权重
+            权重，默认等权
         factorname : str
             因子和的名称  默认名称为'factorSum'
         Returns
@@ -1438,6 +1799,10 @@ class dataProcess():
         '''
         if(addList==''):
             addList=getfactorname(self.dataBase['v'])
+        if(weight_list==''):
+            weight_list=pd.Series(index=addList)
+            weight_list=1
+        weight=weight/weight.sum()
         facSum=0
         for fac_name in addList:
             tmp=self.dataBase['v'].pivot('time','code',fac_name)
@@ -1446,7 +1811,7 @@ class dataProcess():
         self.getData(facSum)
         
     #仍待完善    
-    def factorFillNA(self, factor_list='', freq='month', method='median', window=12):
+    def factorFillNA(self, DF, factor_list='', freq='month', method='median', window=12):
         '''
 
         Parameters
